
/****************************************************************************
 * NCSA HDF                                                                 *
 * Scientific Data Technologies                                             *
 * National Center for Supercomputing Applications                          *
 * University of Illinois at Urbana-Champaign                               *
 * 605 E. Springfield, Champaign IL 61820                                   *
 *                                                                          *
 * For conditions of distribution and use, see the accompanying             *
 * hdf/COPYING f.                                                           *
 *                                                                          *
 ****************************************************************************/


#include "H5HL.h"

#include <stdlib.h>


/*-------------------------------------------------------------------------
 * Table API test
 *
 * Functions tested:
 *
 * H5TBmake_table 
 * H5TBinsert_field
 * H5TBread_table
 *
 *-------------------------------------------------------------------------
 */




#define NFIELDS  5
#define NRECORDS 8

int main( void )
{
 
 typedef struct Particle1 
 {
  char   name[16];
  int    lati;
  int    longi;
  float  pressure;
  double temperature; 
 } Particle1;

 typedef struct Particle2
 {
  char   name[16];
  int    lati;
  int    longi;
  float  pressure;
  double temperature;
  int    new_field;
 } Particle2;
 


 Particle2  dst_buf[NRECORDS];
 
 /* Calculate the size and the offsets of our struct members in memory */
 size_t dst_size1 =  sizeof( Particle1 );
 size_t dst_offset1[NFIELDS] = { HOFFSET( Particle1, name ),
                                  HOFFSET( Particle1, lati ),
                                  HOFFSET( Particle1, longi ),
                                  HOFFSET( Particle1, pressure ),
                                  HOFFSET( Particle1, temperature )};

 size_t dst_size2 =  sizeof( Particle2 );
 size_t dst_offset2[NFIELDS+1] = { HOFFSET( Particle2, name ),
                                  HOFFSET( Particle2, lati ),
                                  HOFFSET( Particle2, longi ),
                                  HOFFSET( Particle2, pressure ),
                                  HOFFSET( Particle2, temperature ),
                                  HOFFSET( Particle2, new_field )};

 size_t dst_sizes2[NFIELDS+1] = { sizeof( dst_buf[0].name),
                                  sizeof( dst_buf[0].lati),
                                  sizeof( dst_buf[0].longi),
                                  sizeof( dst_buf[0].pressure),
                                  sizeof( dst_buf[0].temperature),
                                  sizeof( dst_buf[0].new_field)};
   
 /* Define an array of Particles */
	Particle1  p_data[NRECORDS] = { {"zero",0,0, 0.0f, 0.0},
	{"one",10,10, 1.0f, 10.0},
	{"two",  20,20, 2.0f, 20.0},
	{"three",30,30, 3.0f, 30.0},
	{"four", 40,40, 4.0f, 40.0},
	{"five", 50,50, 5.0f, 50.0},
	{"six",  60,60, 6.0f, 60.0},
	{"seven",70,70, 7.0f, 70.0}
  };

  /* Define field information */
  const char *field_names[NFIELDS]  = 
		{ "Name","Latitude", "Longitude", "Pressure", "Temperature" };
  hid_t      field_type[NFIELDS];
  hid_t      string_type;
  hid_t      file_id;
  hsize_t    chunk_size = 10;
  int        compress  = 0;
  Particle1  fill_data[1] = { {"no data",-1,-1, -99.0f, -99.0} };
  int        fill_data_new[1] = { -100 };
  int        i;
  hsize_t    position;

  /* Define the inserted field information */
  hid_t      field_type_new = H5T_NATIVE_INT;
  int        data[NRECORDS] = { 0,1,2,3,4,5,6,7 };

  /* Initialize the field type */
  string_type = H5Tcopy( H5T_C_S1 );
  H5Tset_size( string_type, 16 );
  field_type[0] = string_type;
  field_type[1] = H5T_NATIVE_INT;
  field_type[2] = H5T_NATIVE_INT;
  field_type[3] = H5T_NATIVE_FLOAT;
  field_type[4] = H5T_NATIVE_DOUBLE;

  /* Create a new file using default properties. */
  file_id = H5Fcreate( "insert_field.h5", H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT );

/*-------------------------------------------------------------------------
 * Make 2 tables
 *-------------------------------------------------------------------------
 */

 TESTING("inserting fields");


 if ( H5TBmake_table( "Table Title", file_id, "Table1", (hsize_t)NFIELDS, (hsize_t)NRECORDS, dst_size1, 
                       field_names, dst_offset1, field_type, 
                       chunk_size, fill_data, compress, p_data  ) < 0 )
   goto out;

 if ( H5TBmake_table( "Table Title", file_id, "Table2", (hsize_t)NFIELDS, (hsize_t)NRECORDS, dst_size1, 
                       field_names, dst_offset1, field_type, 
                       chunk_size, fill_data, compress, p_data  ) < 0 )
   goto out;

 
/*-------------------------------------------------------------------------
 * Insert field test to Table 2
 *-------------------------------------------------------------------------
 */

 /* Insert the new field at the end of the field list */
 position = NFIELDS;
 if ( H5TBinsert_field( file_id, "Table2", "New Field", field_type_new, position, 
	                       fill_data_new, data ) < 0 )
  goto out;

/*-------------------------------------------------------------------------
 * Read the table
 *-------------------------------------------------------------------------
 */
 
 if ( H5TBread_table( file_id, "Table2", dst_size2, dst_offset2, dst_sizes2, dst_buf ) < 0 )
  goto out;

 /* Compare the extracted table with the original array */
 for( i = 0; i < NRECORDS; i++ ) 
 {
 
   if ( ( strcmp( dst_buf[i].name,  p_data[i].name ) != 0 ) ||
         dst_buf[i].lati          != p_data[i].lati ||
         dst_buf[i].longi         != p_data[i].longi ||
         dst_buf[i].pressure      != p_data[i].pressure ||
         dst_buf[i].temperature   != p_data[i].temperature ||
         dst_buf[i].new_field     != data[i] ) {
   goto out;
  }

  
 }

 

/*-------------------------------------------------------------------------
 * end
 *-------------------------------------------------------------------------
 */
 
 /* Close the file. */
 if ( H5Fclose( file_id ) < 0 )
  goto out;

 PASSED();

 return 0;

out:
 /* Close the file. */
 H5Fclose( file_id );
 H5_FAILED();
 return 1;

}

