
/****************************************************************************
 * NCSA HDF                                                                 *
 * Scientific Data Technologies                                             *
 * National Center for Supercomputing Applications                          *
 * University of Illinois at Urbana-Champaign                               *
 * 605 E. Springfield, Champaign IL 61820                                   *
 *                                                                          *
 * For conditions of distribution and use, see the accompanying             *
 * hdf/COPYING f.                                                           *
 *                                                                          *
 ****************************************************************************/


#include "H5HL.h"

#include <stdlib.h>


/*-------------------------------------------------------------------------
 * Table API test
 *
 * Functions tested:
 *
 * H5TBmake_table 
 * H5TBget_table_info
 * H5TBget_field_info
 *
 *-------------------------------------------------------------------------
 */



#define NFIELDS  5
#define NRECORDS 8

int main( void )
{
 
 typedef struct Particle 
 {
  char   name[16];
  int    lati;
  int    longi;
  float  pressure;
  double temperature; 
 } Particle;

 /* Define an array of Particles */
	Particle  p_data[NRECORDS] = { {"zero",0,0, 0.0f, 0.0},
	{"one",10,10, 1.0f, 10.0},
	{"two",  20,20, 2.0f, 20.0},
	{"three",30,30, 3.0f, 30.0},
	{"four", 40,40, 4.0f, 40.0},
	{"five", 50,50, 5.0f, 50.0},
	{"six",  60,60, 6.0f, 60.0},
	{"seven",70,70, 7.0f, 70.0}
  };

	

 /* Calculate the size and the offsets of our struct members in memory */
 size_t dst_size =  sizeof( Particle );
 size_t dst_offset[NFIELDS] = { HOFFSET( Particle, name ),
                                HOFFSET( Particle, lati ),
                                HOFFSET( Particle, longi ),
                                HOFFSET( Particle, pressure ),
                                HOFFSET( Particle, temperature )};
   

  /* Define field information */
  const char *field_names[NFIELDS]  = { "Name","Latitude", "Longitude", "Pressure", "Temperature" };
  hid_t      field_type[NFIELDS];
  hid_t      string_type;
  hid_t      file_id;
  hsize_t    chunk_size = 10;
  int        *fill_data = NULL;
  int        compress  = 0;
  int        i;
  hsize_t    nfields_out;
  hsize_t    nrecords_out;
  char       **names_out;
  size_t     sizes_out[NFIELDS];
  size_t     offset_out[NFIELDS];
  size_t     size_out;

		

  /* Initialize the field field_type */
  string_type = H5Tcopy( H5T_C_S1 );
  H5Tset_size( string_type, 16 );
  field_type[0] = string_type;
  field_type[1] = H5T_NATIVE_INT;
  field_type[2] = H5T_NATIVE_INT;
  field_type[3] = H5T_NATIVE_FLOAT;
  field_type[4] = H5T_NATIVE_DOUBLE;

  
  /* Create a new file using default properties. */
  file_id = H5Fcreate( "make_table.h5", H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT );


/*-------------------------------------------------------------------------
 * H5TBmake_table 
 *-------------------------------------------------------------------------
 */

 if ( H5TBmake_table( "Table Title", file_id, "Table1",(hsize_t) NFIELDS, (hsize_t)NRECORDS, dst_size, 
                       field_names, dst_offset, field_type, 
                       chunk_size, fill_data, compress, p_data  ) < 0 )
   goto out;


/*-------------------------------------------------------------------------
 * H5TBget_table_info  test
 *-------------------------------------------------------------------------
 */

 TESTING("getting table info");


 /* H5TBget_table_info  */
 if ( H5TBget_table_info ( file_id, "Table1", &nfields_out, &nrecords_out ) < 0 )
  goto out;

 if ( NFIELDS != nfields_out || nrecords_out != NRECORDS  ) {
  goto out;
 }

 /* Alocate space */
 names_out = malloc( sizeof(char*) * (size_t)nfields_out );
 for ( i = 0; i < nfields_out; i++) 
 {
  names_out[i] = malloc( sizeof(char) * 255 );
 }

 PASSED();
 
/*-------------------------------------------------------------------------
 * H5TBget_field_info test
 *-------------------------------------------------------------------------
 */

 TESTING("getting field info");


 /* Get field info */
 if ( H5TBget_field_info( file_id, "Table1", names_out, sizes_out, offset_out, &size_out ) < 0 )
  goto out;

 for ( i = 0; i < NFIELDS; i++) 
 {
  if ( strcmp( field_names[i], names_out[i] ) != 0 ) {
   goto out;
  }
 
 }

    
/*-------------------------------------------------------------------------
 * end
 *-------------------------------------------------------------------------
 */
 
 /* Close the file. */
 H5Fclose( file_id );

 /* Release resources. */
 for ( i = 0; i < NFIELDS; i++) 
 {
  free ( names_out[i] );
 }
 free ( names_out );
   

 PASSED();

 return 0;

out:
 /* Close the file. */
 H5Fclose( file_id );
 H5_FAILED();
 return 1;

}

